#include <iostream>
#include <zmq.hpp>
#include <sys/time.h>

#define LOOP_NB	150

using namespace std;

int main(int argc,char *argv[])
{
	int ret = 0;

	if (argc != 2)
	{
		cerr << argv[0] << " usage = " << argv[0] << " <port number>" << endl;
		return -1;
	}

	string port(argv[1]);
	string first_message("tango://kidiboo.esrf.fr:10000/dserver/devtest/api.change");
	int second_part = 0;
	string third_message("This is a long long long long long long long long long long message");
	string forth_message("Another part of the multi-part message");

	try
	{

//
// Create a zmq context with 1 IO thread, create a ZMQ_PUB socket
// and bind it to one endpoint
//

		zmq::context_t context (1);

		zmq::socket_t publisher (context, ZMQ_PUB);

		string endpoint("tcp://*:");
		endpoint = endpoint + port;

//
// Set the publisher socket HWM
//

		int hwm = 10;
        publisher.setsockopt(ZMQ_SNDHWM,&hwm,sizeof(hwm));

		publisher.bind(endpoint.c_str());

//
// Send multipart message
//

		cout << "Hit a key to send messages";
		char key;
		cin >> key;

		for (int i = 0;i < LOOP_NB;i++)
		{
			zmq::message_t message1(first_message.size());
			::memcpy(message1.data(),first_message.data(),first_message.size());

			zmq::message_t message2(sizeof(int));
			::memcpy(message2.data(),&second_part,sizeof(int));
			second_part++;

			zmq::message_t message3(third_message.size());
			::memcpy(message3.data(),third_message.data(),third_message.size());
			
			zmq::message_t message4(forth_message.size());
			::memcpy(message4.data(),forth_message.data(),forth_message.size());

			publisher.send(message1,ZMQ_SNDMORE);
			publisher.send(message2,ZMQ_SNDMORE);
			publisher.send(message3,ZMQ_SNDMORE);
			publisher.send(message4,0);

			struct timeval start;
			gettimeofday(&start,NULL);
			cout << "Publishing msg " << i << " at " << start.tv_sec << " sec and " << start.tv_usec << " usec" << endl;
		}

		sleep(60);

	}
	catch (zmq::error_t &e)
	{
		cerr << "A zmq exception" << endl;
		cerr << "Error code = " << e.num() << endl;
		cerr << "Error message = " << e.what() << endl;

		ret = -1;
	}
	catch (...)
	{
		cerr << "Gasp an unknown exception" << endl;
		ret = -1;
	}

	return ret;
}

